@@ -3,7 +3,7 @@ from __future__ import annotations
 from typing import TYPE_CHECKING, Any, NamedTuple
 
 from ..actions import ActionsPageResult, BoundAction, ResourceActionsClient
-from ..core import BoundModelBase, ClientEntityBase, Meta
+from ..core import BoundModelBase, Meta, ResourceClientBase
 from ..locations import BoundLocation
 from .domain import CreateFloatingIPResponse, FloatingIP
 
@@ -13,24 +13,25 @@ if TYPE_CHECKING:
     from ..servers import BoundServer, Server
 
 
-class BoundFloatingIP(BoundModelBase):
+class BoundFloatingIP(BoundModelBase, FloatingIP):
     _client: FloatingIPsClient
 
     model = FloatingIP
 
     def __init__(self, client: FloatingIPsClient, data: dict, complete: bool = True):
+        # pylint: disable=import-outside-toplevel
         from ..servers import BoundServer
 
         server = data.get("server")
         if server is not None:
             data["server"] = BoundServer(
-                client._client.servers, {"id": server}, complete=False
+                client._parent.servers, {"id": server}, complete=False
             )
 
         home_location = data.get("home_location")
         if home_location is not None:
             data["home_location"] = BoundLocation(
-                client._client.locations, home_location
+                client._parent.locations, home_location
             )
 
         super().__init__(client, data, complete)
@@ -54,7 +55,13 @@ class BoundFloatingIP(BoundModelBase):
                 Specifies how many results are returned by page
         :return: (List[:class:`BoundAction <hcloud.actions.client.BoundAction>`], :class:`Meta <hcloud.core.domain.Meta>`)
         """
-        return self._client.get_actions_list(self, status, sort, page, per_page)
+        return self._client.get_actions_list(
+            self,
+            status=status,
+            sort=sort,
+            page=page,
+            per_page=per_page,
+        )
 
     def get_actions(
         self,
@@ -69,7 +76,7 @@ class BoundFloatingIP(BoundModelBase):
                Specify how the results are sorted. Choices: `id` `id:asc` `id:desc` `command` `command:asc` `command:desc` `status` `status:asc` `status:desc` `progress` `progress:asc` `progress:desc` `started` `started:asc` `started:desc` `finished` `finished:asc` `finished:desc`
         :return: List[:class:`BoundAction <hcloud.actions.client.BoundAction>`]
         """
-        return self._client.get_actions(self, status, sort)
+        return self._client.get_actions(self, status=status, sort=sort)
 
     def update(
         self,
@@ -87,7 +94,9 @@ class BoundFloatingIP(BoundModelBase):
                New Name to set
         :return: :class:`BoundFloatingIP <hcloud.floating_ips.client.BoundFloatingIP>`
         """
-        return self._client.update(self, description, labels, name)
+        return self._client.update(
+            self, description=description, labels=labels, name=name
+        )
 
     def delete(self) -> bool:
         """Deletes a Floating IP. If it is currently assigned to a server it will automatically get unassigned.
@@ -103,7 +112,7 @@ class BoundFloatingIP(BoundModelBase):
                If true, prevents the Floating IP from being deleted
         :return: :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
-        return self._client.change_protection(self, delete)
+        return self._client.change_protection(self, delete=delete)
 
     def assign(self, server: Server | BoundServer) -> BoundAction:
         """Assigns a Floating IP to a server.
@@ -112,7 +121,7 @@ class BoundFloatingIP(BoundModelBase):
                Server the Floating IP shall be assigned to
         :return: :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
-        return self._client.assign(self, server)
+        return self._client.assign(self, server=server)
 
     def unassign(self) -> BoundAction:
         """Unassigns a Floating IP, resulting in it being unreachable. You may assign it to a server again at a later time.
@@ -130,16 +139,16 @@ class BoundFloatingIP(BoundModelBase):
                Hostname to set as a reverse DNS PTR entry, will reset to original default value if `None`
         :return: :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
-        return self._client.change_dns_ptr(self, ip, dns_ptr)
+        return self._client.change_dns_ptr(self, ip=ip, dns_ptr=dns_ptr)
 
 
 class FloatingIPsPageResult(NamedTuple):
     floating_ips: list[BoundFloatingIP]
-    meta: Meta | None
+    meta: Meta
 
 
-class FloatingIPsClient(ClientEntityBase):
-    _client: Client
+class FloatingIPsClient(ResourceClientBase):
+    _base_url = "/floating_ips"
 
     actions: ResourceActionsClient
     """Floating IPs scoped actions client
@@ -149,7 +158,7 @@ class FloatingIPsClient(ClientEntityBase):
 
     def __init__(self, client: Client):
         super().__init__(client)
-        self.actions = ResourceActionsClient(client, "/floating_ips")
+        self.actions = ResourceActionsClient(client, self._base_url)
 
     def get_actions_list(
         self,
@@ -182,12 +191,12 @@ class FloatingIPsClient(ClientEntityBase):
         if per_page is not None:
             params["per_page"] = per_page
         response = self._client.request(
-            url=f"/floating_ips/{floating_ip.id}/actions",
+            url=f"{self._base_url}/{floating_ip.id}/actions",
             method="GET",
             params=params,
         )
         actions = [
-            BoundAction(self._client.actions, action_data)
+            BoundAction(self._parent.actions, action_data)
             for action_data in response["actions"]
         ]
         return ActionsPageResult(actions, Meta.parse_meta(response))
@@ -221,7 +230,7 @@ class FloatingIPsClient(ClientEntityBase):
         :param id: int
         :return: :class:`BoundFloatingIP <hcloud.floating_ips.client.BoundFloatingIP>`
         """
-        response = self._client.request(url=f"/floating_ips/{id}", method="GET")
+        response = self._client.request(url=f"{self._base_url}/{id}", method="GET")
         return BoundFloatingIP(self, response["floating_ip"])
 
     def get_list(
@@ -254,9 +263,7 @@ class FloatingIPsClient(ClientEntityBase):
         if name is not None:
             params["name"] = name
 
-        response = self._client.request(
-            url="/floating_ips", method="GET", params=params
-        )
+        response = self._client.request(url=self._base_url, method="GET", params=params)
         floating_ips = [
             BoundFloatingIP(self, floating_ip_data)
             for floating_ip_data in response["floating_ips"]
@@ -286,13 +293,13 @@ class FloatingIPsClient(ClientEntityBase):
                Used to get Floating IP by name.
         :return: :class:`BoundFloatingIP <hcloud.floating_ips.client.BoundFloatingIP>`
         """
-        return self._get_first_by(name=name)
+        return self._get_first_by(self.get_list, name=name)
 
     def create(
         self,
         type: str,
         description: str | None = None,
-        labels: str | None = None,
+        labels: dict[str, str] | None = None,
         home_location: Location | BoundLocation | None = None,
         server: Server | BoundServer | None = None,
         name: str | None = None,
@@ -324,11 +331,11 @@ class FloatingIPsClient(ClientEntityBase):
         if name is not None:
             data["name"] = name
 
-        response = self._client.request(url="/floating_ips", json=data, method="POST")
+        response = self._client.request(url=self._base_url, json=data, method="POST")
 
         action = None
         if response.get("action") is not None:
-            action = BoundAction(self._client.actions, response["action"])
+            action = BoundAction(self._parent.actions, response["action"])
 
         result = CreateFloatingIPResponse(
             floating_ip=BoundFloatingIP(self, response["floating_ip"]), action=action
@@ -362,7 +369,7 @@ class FloatingIPsClient(ClientEntityBase):
             data["name"] = name
 
         response = self._client.request(
-            url=f"/floating_ips/{floating_ip.id}",
+            url=f"{self._base_url}/{floating_ip.id}",
             method="PUT",
             json=data,
         )
@@ -375,7 +382,7 @@ class FloatingIPsClient(ClientEntityBase):
         :return: boolean
         """
         self._client.request(
-            url=f"/floating_ips/{floating_ip.id}",
+            url=f"{self._base_url}/{floating_ip.id}",
             method="DELETE",
         )
         # Return always true, because the API does not return an action for it. When an error occurs a HcloudAPIException will be raised
@@ -398,11 +405,11 @@ class FloatingIPsClient(ClientEntityBase):
             data.update({"delete": delete})
 
         response = self._client.request(
-            url=f"/floating_ips/{floating_ip.id}/actions/change_protection",
+            url=f"{self._base_url}/{floating_ip.id}/actions/change_protection",
             method="POST",
             json=data,
         )
-        return BoundAction(self._client.actions, response["action"])
+        return BoundAction(self._parent.actions, response["action"])
 
     def assign(
         self,
@@ -417,11 +424,11 @@ class FloatingIPsClient(ClientEntityBase):
         :return: :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
         response = self._client.request(
-            url=f"/floating_ips/{floating_ip.id}/actions/assign",
+            url=f"{self._base_url}/{floating_ip.id}/actions/assign",
             method="POST",
             json={"server": server.id},
         )
-        return BoundAction(self._client.actions, response["action"])
+        return BoundAction(self._parent.actions, response["action"])
 
     def unassign(self, floating_ip: FloatingIP | BoundFloatingIP) -> BoundAction:
         """Unassigns a Floating IP, resulting in it being unreachable. You may assign it to a server again at a later time.
@@ -430,10 +437,10 @@ class FloatingIPsClient(ClientEntityBase):
         :return: :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
         response = self._client.request(
-            url=f"/floating_ips/{floating_ip.id}/actions/unassign",
+            url=f"{self._base_url}/{floating_ip.id}/actions/unassign",
             method="POST",
         )
-        return BoundAction(self._client.actions, response["action"])
+        return BoundAction(self._parent.actions, response["action"])
 
     def change_dns_ptr(
         self,
@@ -451,8 +458,8 @@ class FloatingIPsClient(ClientEntityBase):
         :return: :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
         response = self._client.request(
-            url=f"/floating_ips/{floating_ip.id}/actions/change_dns_ptr",
+            url=f"{self._base_url}/{floating_ip.id}/actions/change_dns_ptr",
             method="POST",
             json={"ip": ip, "dns_ptr": dns_ptr},
         )
-        return BoundAction(self._client.actions, response["action"])
+        return BoundAction(self._parent.actions, response["action"])
